@@ -22,6 +22,8 @@
 #include <algorithm>
 #include <memory>
 
+#include "absl/container/inlined_vector.h"
+
 namespace grpc_core {
 
 template <class K, class V = void>
@@ -32,8 +34,12 @@ class AVL {
   AVL Add(K key, V value) const {
     return AVL(AddKey(root_, std::move(key), std::move(value)));
   }
-  AVL Remove(const K& key) const { return AVL(RemoveKey(root_, key)); }
-  const V* Lookup(const K& key) const {
+  template <typename SomethingLikeK>
+  AVL Remove(const SomethingLikeK& key) const {
+    return AVL(RemoveKey(root_, key));
+  }
+  template <typename SomethingLikeK>
+  const V* Lookup(const SomethingLikeK& key) const {
     NodePtr n = Get(root_, key);
     return n ? &n->kv.second : nullptr;
   }
@@ -50,7 +56,36 @@ class AVL {
     ForEachImpl(root_.get(), std::forward<F>(f));
   }
 
-  bool SameIdentity(AVL avl) const { return root_ == avl.root_; }
+  bool SameIdentity(const AVL& avl) const { return root_ == avl.root_; }
+
+  bool operator==(const AVL& other) const {
+    Iterator a(root_);
+    Iterator b(other.root_);
+    for (;;) {
+      Node* p = a.current();
+      Node* q = b.current();
+      if (p == nullptr) return q == nullptr;
+      if (q == nullptr) return false;
+      if (p->kv != q->kv) return false;
+      a.MoveNext();
+      b.MoveNext();
+    }
+  }
+
+  bool operator<(const AVL& other) const {
+    Iterator a(root_);
+    Iterator b(other.root_);
+    for (;;) {
+      Node* p = a.current();
+      Node* q = b.current();
+      if (p == nullptr) return q != nullptr;
+      if (q == nullptr) return false;
+      if (p->kv < q->kv) return true;
+      if (p->kv != q->kv) return false;
+      a.MoveNext();
+      b.MoveNext();
+    }
+  }
 
  private:
   struct Node;
@@ -68,6 +103,32 @@ class AVL {
   };
   NodePtr root_;
 
+  class Iterator {
+   public:
+    explicit Iterator(const NodePtr& root) {
+      auto* n = root.get();
+      while (n != nullptr) {
+        stack_.push_back(n);
+        n = n->left.get();
+      }
+    }
+    Node* current() const { return stack_.empty() ? nullptr : stack_.back(); }
+    void MoveNext() {
+      auto* n = stack_.back();
+      stack_.pop_back();
+      if (n->right != nullptr) {
+        n = n->right.get();
+        while (n != nullptr) {
+          stack_.push_back(n);
+          n = n->left.get();
+        }
+      }
+    }
+
+   private:
+    absl::InlinedVector<Node*, 8> stack_;
+  };
+
   explicit AVL(NodePtr root) : root_(std::move(root)) {}
 
   template <class F>
@@ -86,7 +147,8 @@ class AVL {
                                   1 + std::max(Height(left), Height(right)));
   }
 
-  static NodePtr Get(const NodePtr& node, const K& key) {
+  template <typename SomethingLikeK>
+  static NodePtr Get(const NodePtr& node, const SomethingLikeK& key) {
     if (node == nullptr) {
       return nullptr;
     }
@@ -198,7 +260,8 @@ class AVL {
     return node;
   }
 
-  static NodePtr RemoveKey(const NodePtr& node, const K& key) {
+  template <typename SomethingLikeK>
+  static NodePtr RemoveKey(const NodePtr& node, const SomethingLikeK& key) {
     if (node == nullptr) {
       return nullptr;
     }
